@@ -51,12 +51,16 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
                                 soil.child = c("Metadata", "Water", "InitialWater",
                                           "Chemical", "Physical", "Analysis", "SoilWater",
                                           "InitialN", "CERESSoilTemperature", "Sample",
+                                          "Solute", "NO3", "NH4", "Urea",
                                           "Nutrient", "Organic", "Swim3"),
                                 parm = NULL,
                                 digits = 3,
-                                root){
+                                root = NULL){
   #### Beginning of function ----
-  .check_apsim_name(file)
+  if(isFALSE(apsimx::apsimx.options$allow.path.spaces)){
+    .check_apsim_name(file)
+    .check_apsim_name(normalizePath(src.dir))
+  }
   
   file.names <- dir(path = src.dir, pattern=".apsimx$", ignore.case=TRUE)
   
@@ -240,8 +244,8 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
     }
   }
 
-  if((length(fcsn) > 1 || !missing(root)) && find.root){
-    if(missing(root)){
+  if((length(fcsn) > 1 || !is.null(root)) && find.root){
+    if(is.null(root)){
       cat("Simulation structure: \n")
       str_list(apsimx_json)
       stop("more than one simulation found and no root node label has been specified \n select one of the children names above", call. = FALSE)   
@@ -398,22 +402,42 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
     }else{
       ## Pick which soil component we want to look at
       ## Which is not 'Metadata"
-      if(soil.child == "InitialWater") soil.child <- "Water"
-      
-      wsc <- grep(soil.child, soil.children.names)
-      
+      ## If "InitialWater" is present then use it
+      wsiw <- "InitialWater" %in% soil.children.names
+      if(soil.child %in% c("Water", "InitialWater")){
+        if(soil.child == "InitialWater" && isFALSE(wsiw)){
+          wsiw2 <- grep("initial water", soil.children.names, ignore.case = TRUE)
+          if(length(wsiw2) > 0){
+            soil.child <-  grep("initial water", soil.children.names, ignore.case = TRUE, value = TRUE)
+          }else{
+            wsiw3 <- grep("^Water", soil.children.names)
+            if(length(wsiw3) > 0){
+              soil.child <- "Water"
+            }else{
+              soil.child <- "InitialWater"                          
+            }
+          }
+        }        
+      }
+ 
+      if(soil.child != "Solute"){
+        wsc <- which(soil.child == soil.children.names)
+      }else{
+        wsc <- which(soil.children.names %in% c("NO3", "NH4", "Urea"))
+      }
+
       if(soil.child == "Water" && length(wsc) == 2) wsc <- wsc[2]
       
       if(length(wsc) == 0) stop("soil.child likely not present")
       
       selected.soil.node.child <- soil.node[[1]]$Children[wsc]
     }
-    
+    ## if(soil.child == "InitialWater") browser()
     ## For some variables now it is the time to print
     ## The code below is not strictly needed but it is here
     ## in case I need a second level of soil in the future
     first.level.soil <- c("Water", "Physical",
-                          "Chemical", "Analysis", "InitialWater",
+                          "Chemical", "Analysis", "InitialWater", "Initial water", "initial water",
                           "InitialN", "SoilWater", "Analysis",
                           "CERESSoilTemperature", "Organic", "Swim3")
     if(soil.child %in% first.level.soil){
@@ -434,7 +458,7 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
       d3.col.nms <- NULL
       
       for(ii in cnms){
-        
+
         tmp <- selected.soil.node.child[[1]][ii][[1]]
         
         if(ii %in% c("Crop LL", "Crop KL", "Crop XF")){
@@ -458,7 +482,7 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
           if(!ii %in% c("Crop LL", "Crop KL", "Crop XF")){
             col.nms <- c(col.nms, ii)
             vals <- as.vector(unlist(tmp))
-            soil.d2 <- cbind(soil.d2, vals)                        
+            soil.d2 <- cbind(soil.d2, vals)
           }
         }
       }
@@ -540,6 +564,85 @@ extract_data_apsimx <- function(file = "", src.dir = ".",
     }
   }
   
+  second.level.soil <- c("Solute", "NO3", "NH4", "Urea")
+  
+  if(soil.child %in% second.level.soil){
+    
+    soil.d2 <- NULL
+    soil.d3 <- NULL
+    enms <- c("IncludeInDocumentation", "Enabled", "ReadOnly", "Children", "Name", "$type")
+    ssnc <- selected.soil.node.child
+    
+    if(soil.child == "Solute"){
+      
+      solutes <- sapply(ssnc, function(x) x$Name)
+      
+      parm.path <- paste0(parm.path.2.1, ".", "Solute")
+      
+      if(missing(parm)){
+        ## cat("Solutes:", solutes, "\n")
+        ## Do I need to do anything here?
+      }else{
+        ret_lst <- vector('list', length = length(solutes))
+        names(ret_lst) <- solutes
+        sel.parm <- grep(parm[[1]], solutes)
+        if(length(sel.parm) == 0)
+          stop("'parm' should be one of: ", paste(solutes, collapse = ", "), call. = FALSE)
+        solutes <- parm[[1]]
+      }
+      
+      ret_lst <- vector('list', length = length(solutes))
+      names(ret_lst) <- solutes
+      
+      for(j in seq_along(solutes)){
+        ## cat("\nSolute:", solutes[j], "\n")
+        ssnc.solute <- ssnc[[j]]
+        if(length(unlist(ssnc.solute$Thickness)) != length(unlist(ssnc.solute$InitialValues))){
+          cat("Length of Thickness:", length(unlist(ssnc.solute$Thickness)), "\n")
+          cat("Length of InitialValues:", length(unlist(ssnc.solute$InitialValues)), "\n")
+          stop("Length of 'Thickness' does not match length of 'InitialValues'", call. = FALSE)
+        }
+        soil.d1 <- data.frame(Thickness = unlist(ssnc.solute$Thickness),
+                              InitialValues = unlist(ssnc.solute$InitialValues))
+        if(!is.null(soil.d1)){
+          if(is.null(parm)){
+            ret_lst[[j]][["first"]] <- soil.d1
+          }else{
+            if(length(parm) == 1){
+              if(parm %in% c("Thickness", "InitialValues")){
+                ret_lst[[j]][["first"]] <- soil.d1[, parm, drop = FALSE]
+              }else{
+                ret_lst[[j]][["first"]] <- soil.d1
+              }
+            }
+          }
+        } 
+        
+        cnms <- setdiff(names(ssnc.solute), enms)        
+        for(k in cnms){
+          if(k %in% c("Thickness", "InitialValues")) next
+          tmp <- ssnc.solute[k][[1]]
+          if(is.null(tmp)) tmp <- ""
+          soil.d2 <- rbind(soil.d2,
+                           data.frame(parm = k, value = as.character(tmp)))
+        }
+        if(missing(parm) || length(parm) == 1){
+          if(!is.null(soil.d2)){
+            ret_lst[[j]][["second"]] <- soil.d2
+          } 
+        }else{
+          if(is.list(parm) && is.numeric(parm[[2]])){
+            soil.d2.s <- soil.d2[parm[[2]], , drop = FALSE]
+            if(!is.null(soil.d2)){
+              ret_lst[[j]][["second"]] <- soil.d2.s
+            }            
+          }
+        }
+        soil.d2 <- NULL
+      }
+    }
+  }
+  
   if(node == "SurfaceOrganicMatter"){
     ## Which is 'SurfaceOrganicMatter'
     wsomn <- grepl("Models.Surface.SurfaceOrganicMatter", core.zone.node)
